@@ -15,10 +15,11 @@ except ImportError:
 
 import base64
 from urlparse import urljoin, urlparse
+from datetime import datetime, timedelta
 
 from .exceptions import Forbidden, GeneralException, InternalError, InvalidData, NotFound
 from .response import Codes
-from .util import quote, hash
+from .util import quote, hash, tzutc
 
 
 class Vingd:
@@ -29,6 +30,9 @@ class Vingd:
     URL_ENDPOINT_SANDBOX = "https://api.vingd.com/sandbox/broker/v1"
     URL_FRONTEND_SANDBOX = "http://www.sandbox.vingd.com"
     
+    EXP_ORDER = {'minutes': 15}
+    EXP_VOUCHER = {'days': 7}
+    
     api_key = None
     api_secret = None
     api_endpoint = URL_ENDPOINT
@@ -164,6 +168,7 @@ class Vingd:
                 token = {
                     "object": <object_name>,
                     "huid": <hashed_user_id_bound_to_seller> / None,
+                    "context": <order_context> / None,
                     ...
                 }
             
@@ -184,6 +189,8 @@ class Vingd:
                   and user's privacy is guaranteed. Also, note that the value of
                   ``huid`` **will be** ``null`` iff buyer chose anonymous
                   purchase.
+                * ``context`` is an arbitrary purchase context defined when
+                  creating order.
         
         :raises GeneralException:
         :raises Forbidden:
@@ -231,7 +238,7 @@ class Vingd:
             json.dumps({'transferid': transferid})
         )
     
-    def create_order(self, oid, price, expires):
+    def create_order(self, oid, price, context=None, expires=None):
         """
         CREATES a single order for object ``oid``, with price set to ``price``
         and validity until ``expires``.
@@ -243,9 +250,17 @@ class Vingd:
         :param price:
             Vingd amount (in cents) the user/buyer shall be charged upon
             successful purchase.
-        :type expires: ``datetime``
+        :type context: ``string``
+        :param context:
+            Purchase (order-related) context. Retrieved upon purchase
+            verification.
+        :type expires: ``datetime``/``dict``
         :param expires:
-            Order expiry timestamp.
+            Order expiry timestamp, absolute (``datetime``) or relative
+            (``dict``). Valid keys for relative expiry timestamp dictionary are
+            same as keyword arguments for `datetime.timedelta` (``days``,
+            ``seconds``, ``minutes``, ``hours``, ``weeks``). Default:
+            `Vingd.EXP_ORDER`.
         
         :rtype: ``dict``
         :returns:
@@ -254,6 +269,7 @@ class Vingd:
                 order = {
                     'id': <order_id>,
                     'expires': <order_expiry>,
+                    'context': <purchase_context>,
                     'object': {
                         'id': <oid>,
                         'price': <amount_in_cents>
@@ -268,13 +284,20 @@ class Vingd:
         :resource: ``objects/<oid>/orders/``
         :access: authorized users
         """
+        if expires is None:
+            expires = self.EXP_ORDER
+        if isinstance(expires, dict):
+            expires = datetime.now(tzutc())+timedelta(**expires)
         orders = self.request('post', 'objects/%d/orders/' % int(oid), json.dumps({
-            'price': price, 'order_expires': expires.isoformat()
+            'price': price,
+            'order_expires': expires.isoformat(),
+            'context': context
         }))
         orderid = self._extract_id_from_batch_response(orders)
         return {
             'id': orderid,
             'expires': expires.isoformat(),
+            'context': context,
             'object': {
                 'id': oid,
                 'price': price
@@ -502,7 +525,7 @@ class Vingd:
             'description': description
         }))
     
-    def create_voucher(self, amount, expires, message='', gid=None):
+    def create_voucher(self, amount, expires=None, message='', gid=None):
         """
         CREATES a new preallocated voucher with ``amount`` vingd cents reserved
         until ``expires``.
@@ -510,9 +533,13 @@ class Vingd:
         :type amount: ``bigint``
         :param amount:
             Voucher amount in vingd cents.
-        :type expires: ``datetime``
+        :type expires: ``datetime``/``dict``
         :param expires:
-            Voucher expiry timestamp.
+            Voucher expiry timestamp, absolute (``datetime``) or relative
+            (``dict``). Valid keys for relative expiry timestamp dictionary are
+            same as keyword arguments for `datetime.timedelta` (``days``,
+            ``seconds``, ``minutes``, ``hours``, ``weeks``). Default:
+            `Vingd.EXP_VOUCHER`.
         :type message: ``string``
         :param message:
             Short message displayed to user when she redeems the voucher on
@@ -549,6 +576,10 @@ class Vingd:
         :resource: ``vouchers/``
         :access: authorized users (ACL flag: ``voucher.add``)
         """
+        if expires is None:
+            expires = self.EXP_VOUCHER
+        if isinstance(expires, dict):
+            expires = datetime.now(tzutc())+timedelta(**expires)
         voucher = self.request('post', 'vouchers/', json.dumps({
             'amount': amount,
             'until': expires.isoformat(),
